package rition.backend.api.v1.panel;

import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;
import rition.backend.api.v1.dto.request.MetricDataRequest;
import rition.backend.api.v1.dto.response.MetricDataResponse;
import rition.backend.api.v1.dto.response.Response;
import rition.common.data.dto.service.MetricDataDto;
import rition.common.data.entity.MetricRecordEntity;
import rition.common.data.enums.Constants;
import rition.service.panel.MetricService;

import java.time.Instant;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/panel/metrics")
public class MetricsViewController {

    private final MetricService metricService;
    private final RedisTemplate<String, Object> redisTemplate;

    public MetricsViewController(MetricService metricService, RedisTemplate<String, Object> redisTemplate) {
        this.metricService = metricService;
        this.redisTemplate = redisTemplate;
    }

    @PostMapping("/list")
    public Response<List<MetricDataResponse>> getMetrics(@RequestBody MetricDataRequest metricDataRequest) {
        // 如果未给定范围参数，则使用最近一小时的数据
        if (metricDataRequest.getStart() == null || metricDataRequest.getEnd() == null) {
            List<Object> recentMetricDataList =
                    redisTemplate.opsForList().range(
                            Constants.RedisKeys.RECENT_METRIC_CACHE, 0, Constants.MAX_METRIC_CACHE_NUM
                    );
            if (recentMetricDataList == null) {
                return Response.success(new ArrayList<>());
            }

            List<MetricDataResponse> responseList = new ArrayList<>(recentMetricDataList.size());
            for (Object recentMetricData : recentMetricDataList) {
                var data = (MetricDataDto) recentMetricData;
                responseList.add(MetricDataResponse.builder()
                        .metricData(data.getData())
                        .time(data.getTimestamp())
                        .build());
            }

            return Response.success(responseList);
        }

        var entityResult = metricService.getMetricDataRange(
                metricDataRequest.getInstanceId(),
                this.filterMetricItems(metricDataRequest.getMetricItems()),
                Instant.ofEpochMilli(metricDataRequest.getStart()),
                Instant.ofEpochMilli(metricDataRequest.getEnd())
        );

        // 转换数据
        List<MetricDataResponse> responseList = new ArrayList<>(entityResult.size());
        for (MetricRecordEntity entity : entityResult) {
            responseList.add(MetricDataResponse.builder()
                    .metricData(entity.getMetricData())
                    .time(entity.getTime().toEpochMilli())
                    .build()
            );
        }

        return Response.success(responseList);
    }


    private static final HashSet<String> allowedMetricItems = new HashSet<>(22);

    static {
        allowedMetricItems.add("up");
        allowedMetricItems.add("node_load5");
        allowedMetricItems.add("node_sockstat_TCP_tw");
        allowedMetricItems.add("node_cpu_seconds_total");
        allowedMetricItems.add("node_memory_Cached_bytes");
        allowedMetricItems.add("node_memory_Buffers_bytes");
        allowedMetricItems.add("node_memory_MemFree_bytes");
        allowedMetricItems.add("node_disk_read_bytes_total");
        allowedMetricItems.add("node_filesystem_free_bytes");
        allowedMetricItems.add("node_filesystem_size_bytes");
        allowedMetricItems.add("node_memory_MemTotal_bytes");
        allowedMetricItems.add("node_netstat_Tcp_CurrEstab");
        allowedMetricItems.add("node_filesystem_avail_bytes");
        allowedMetricItems.add("node_disk_written_bytes_total");
        allowedMetricItems.add("node_disk_reads_completed_total");
        allowedMetricItems.add("node_network_receive_drop_total");
        allowedMetricItems.add("node_disk_writes_completed_total");
        allowedMetricItems.add("node_network_receive_bytes_total");
        allowedMetricItems.add("node_network_transmit_drop_total");
        allowedMetricItems.add("node_network_transmit_bytes_total");
        allowedMetricItems.add("node_network_receive_packets_total");
        allowedMetricItems.add("node_network_transmit_packets_total");
    }

    private List<String> filterMetricItems(List<String> metricItems) {
        List<String> filteredMetricItems = new ArrayList<>(metricItems.size());
        for (String metricItem : metricItems) {
            if (allowedMetricItems.contains(metricItem)) {
                filteredMetricItems.add(metricItem);
            } else {
                log.warn("[MetricService]: 请求的指标值含有非允许的指标值：{}", metricItem);
            }
        }

        return filteredMetricItems;
    }
}
